###
## SecClabCa
## Copyright (C) 2003 SecClab group (mail. lfp70@hotmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public Licence as published by
# the Free Software Foundation; eihter version 2 of the License, or
# (at your option) any later version.
#
# This program is distributes in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
##

from Globals import DTMLFile,InitializeClass,HTML
from AccessControl import ClassSecurityInfo
from OFS.Folder import Folder
from App.Common import package_home

import OpensslWrapper
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
from OFS.Image import Image
import string
import shutil
import tempfile
#import traceback
import sys
import os
import types
import exceptions
import time



MessageDialog = HTML("""
<HTML>
<HEAD>
<TITLE><dtml-var title></TITLE>
</HEAD>
<BODY BGCOLOR="#FFFFFF">
<FORM ACTION="<dtml-var action>" METHOD="GET" <dtml-if
 target>TARGET="<dtml-var target>"</dtml-if>>
 <dtml-in cgi_values prefix="lis">
   <input type="hidden" name="<dtml-var expr="lis_item[0]">" value="<dtml-var expr="lis_item[1]">">
 </dtml-in>
<TABLE BORDER="0" WIDTH="100%" CELLPADDING="10">
<TR>
  <TD VALIGN="TOP">
  <BR>
  <CENTER><B><FONT SIZE="+6" COLOR="#77003B">!</FONT></B></CENTER>
  </TD>
  <TD VALIGN="TOP">
  <BR><BR>
  <dtml-in expr="_.string.split(message,'\\n')">
    <dtml-var sequence-item><br>
  </dtml-in>
  </TD>
</TR>
<TR>
  <TD VALIGN="TOP">
  </TD>
  <TD VALIGN="TOP">
  <CENTER>
  <INPUT TYPE="SUBMIT" VALUE="   Ok   ">
  </CENTER>
  </TD>
</TR>
</TABLE>
</FORM>
</BODY></HTML>""", target='', action='manage_main', title='Changed', cgi_values={})
#"


manage_addSecClabCaForm = DTMLFile('dtml/manage_addSecClabCaForm',globals())

    
def manage_addSecClabCa (self, id,
                         title='',
                         cadir='',
                         REQUEST=None):
    """ Add a new SecClabCa to current ObjectManager """

    ca = SecClabCa(id,title,cadir)
    self._setObject(id,ca)
    ca = self._getOb(id)
    ca._templateReset(package_home(globals()))

    ca.manage_addProduct['SecClabCa'].manage_addSecClabAccess('SecClab Access','')
    
    if REQUEST:
        return self.manage_main(self,REQUEST,update_menu=1)



class SecClabCa(Folder):
    """ a folder with CA-managing-features (openssl based) """

    security = ClassSecurityInfo()
    
    meta_type = 'SecClab Certification Authority'
    title = ''
    dir = ''
    rafolder = ''

    _properties=(
        {'id':'title',      'type':'string', 'mode':'w'},
        {'id':'dir',        'type':'string', 'mode':'w'},
        {'id':'ca_generated','type':'boolean','mode':'w'},
        {'id':'rafolder',    'type':'string', 'mode':'w'},
        )

    security.declarePublic('manage_secClabSetPassword')
    def manage_secClabSetPassword(self,REQUEST):
        """ Manage """
        objects = self.objectValues('SecClab Access')
        resp = REQUEST['RESPONSE']
        if OpensslWrapper.CheckCAPassword(self.dir,REQUEST['password']) == 1:
            REQUEST.SESSION[objects[0].session_variable] = REQUEST['password']
            return MessageDialog(
                title  ='You have been succesfully logged',
                message="You have been succesfully logged",
                action ='../')
        else:
            return MessageDialog(
                title  ='Error in password',
                message="Invalid password",
                action ='javascript:history.go(-1);')

    security.declarePublic('manage_secClabResetPassword')
    def manage_secClabResetPassword(self,REQUEST):
        """ manage """
        objects = self.objectValues('SecClab Access')
        del REQUEST.SESSION[objects[0].session_variable]
        return MessageDialog(
            title  ='Session has been closed',
            message="Session has been closed",
            action ='./')
    
    def __init__(self,id,title,dir):
        """ setup a SecClabCa """
        self.id = id
        self.title = title
        self.dir = dir
        self.ca_generated = 0
        self.rafolder = ""

    def _getPassword(self):
        objects = self.objectValues('SecClab Access')
        return self.REQUEST.SESSION[objects[0].session_variable]

    def _setPassword(self,password):
        objects = self.objectValues('SecClab Access')
        self.REQUEST.SESSION[objects[0].session_variable] = password
                
    def SetRaFolder(self,rafolder):
        self.rafolder = rafolder

    def GetRaFolder(self):
        if len(self.rafolder) == 0:
            raise 'ERROR', 'RA folder value not valid. Please contact with administrator site'
        try:
            raobject = self.restrictedTraverse(string.split(self.rafolder,'/'))
        except exceptions.KeyError:
            raise 'ERROR','RA folder not valid. Please contact with administrator site'
        except:
            raise 'ERROR','Please contact with administrator site (%s-%s-%s)' % (sys.exc_type,sys.exc_value,sys.exc_traceback)

        if raobject.meta_type != 'SecClab Registration Authority':
            raise 'ERROR','RA folder is not an RA Object. Please contact with administrator site'
        return raobject
        
    security.declarePublic('manage_generateKeys')
    def manage_generateKeys(self,REQUEST):
        """ Generate keys """
        if self.ca_generated:
            return MessageDialog(
                title  ='Error trying generating keys...',
                message='Certification Authority key had been generetated before',
                action ='keys')

        if REQUEST.form['password'] != REQUEST.form['password2']:
            return MessageDialog(
                title  ='Error trying generating keys...',
                message='Passwords not equal',
                action ='keys')
        if len(REQUEST.form['password']) < 4:
            return MessageDialog(
                title  ='Error trying generating keys...',
                message='Password len less than 4 characters',
                action ='keys')
        
        # OPENSSL CREATE CA
        os.mkdir (self.dir)
        try:
            OpensslWrapper.NewCA(self.dir,REQUEST.form['password'],REQUEST.form)
            self._copyExtensionsTemplates(package_home(globals()))
        except:
            if sys.platform[:3] != "win":
                shutil.rmtree(self.dir)
            raise sys.exc_type,sys.exc_value,sys.exc_traceback
        
        self.ca_generated = 1
        self._setPassword(REQUEST.form['password'])

        get_transaction().commit()
        
        REQUEST['RESPONSE'].redirect(self.absolute_url()+'/keys')
        return

    security.declarePublic('GetCaCertList')
    def GetCaCertList(self):
        """ Get CA Cert list """
        return OpensslWrapper.GetCACertList(self.dir,self._getPassword())

    security.declarePublic('VerifyCaCert')
    def VerifyCaCert(self):
        """ VerifyCaCert """

        return 0,""

    security.declarePublic('manage_addCert')
    def manage_addCert(self,certfile,pem=None,path=None,REQUEST=None):
        """ Add a certificate """
        if not self.ca_generated:
            return MessageDialog(
                title  ='Error trying to import certificate...',
                message='Certification Authority key had been generetated before',
                action ='keys')
        if len(certfile.filename) == 0:
            return MessageDialog(
                title  ='Error trying to import certificate...',
                message='You must input a file',
                action ='keys')

        certificate = certfile.read()
        if len(certificate) == 0:
            return MessageDialog(
                title  ='Error trying to import certificate...',
                message='The file content is null',
                action ='keys')
        # ADD CERTIFICATE

        # REQUEST['RESPONSE'].redirect(self.absolute_url()+'/keys')
        # return
        if pem is not None:
            pem = 1
        else:
            pem = 0
        if path is not None:
            path = 1
        else:
            path = 0
        try:
            ret = OpensslWrapper.ImportCACert(self.dir,certificate,self._getPassword(),pem,path)
        except:
            return MessageDialog(
                 title  ='Error trying to import certificate...',
                 message= "(%s): %s" % (sys.exc_type,sys.exc_value),
                 action ='keys')
        
        return MessageDialog(
                title  ='Certificate successfully imported',
                message= ret,
                action ='keys')
    def _templateFile(self,file,title='',prefix='',type='PageTemplate'):
        f = open("%s/templates/ca/default/%s" % (prefix,file),"rb");data=f.read();f.close()
        if type == 'PageTemplate':
            t = ZopePageTemplate(file,data,"text/html")
            t.pt_setTitle(title)
            self._setObject(file,t)
        elif type == 'Image':
            t = Image(file,title,data)
            self._setObject(file,t)
        else:
            self.manage_addDTMLMethod(file,title,data)
    def _templateReset(self,prefix):
        for file in os.listdir("%s/templates/ca/default" % prefix):
            extension = string.split(file,'.')[-1]
            if extension == 'webprj':
                continue
            elif extension == 'gif':
                self._templateFile(file,file,prefix,type='Image')
            elif extension == 'css':
                self._templateFile(file,file,prefix,type='DTML')
            else:
                self._templateFile(file,file,prefix)

    def _copyExtensionsTemplates(self,prefix):
        for file in os.listdir('%s/CAPolicyTemplates' % prefix):
            name = string.split(file,'.')
            if name[-1] == 'osslcnf':
                OpensslWrapper.AddExtensionTemplate(self.dir,"%s/CAPolicyTemplates/%s" % (prefix,file),string.join(name[:-1],'.'))
                
    security.declarePublic('GetCertificateInfoHtml')
    def GetCertificateInfoHtml(self,own,ident):
        """ Get CertificateInfoHtml """
        cert_info = OpensslWrapper.OpensslGetCertificateInfo(self.dir,self._getPassword(),string.atoi(own),ident);
        lines = string.split(cert_info,'\n')
        for l in range(len(lines)-1):
            if (len(lines[l]) > 0):
                if lines[l][0] == ' ':
                    counter = 1
                    while counter < len(lines[l]):
                        if lines[l][counter] != ' ':
                            break
                        counter = counter + 1
                    for i in range(0,counter,2):
                        lines[l] =  '&nbsp;' + lines[l]
        return string.join(lines,'<br>\n')
        
    security.declarePublic('GetPolicies')
    def GetPolicies(self):
        """ Get CA policies """
        return OpensslWrapper.GetPolicyList(self.dir)
    
    security.declarePublic('GetTemplates')
    def GetTemplates(self):
        """ Get CA templates """
        return OpensslWrapper.GetTemplateList(self.dir)

    security.declarePublic('GetPolicy')
    def GetPolicy(self,REQUEST):
        """ Get a policy by name o create """

        return OpensslWrapper.GetPolicy(self.dir,REQUEST.form['policy'],1)

    security.declarePublic('manage_addPolicy')
    def manage_addPolicy(self,REQUEST):
        """ add new policy """
        if len(string.strip(REQUEST.form['policy'])) == 0:
            return MessageDialog(
                title  ='Error creating policy...',
                message="Policy name not valid",
                action ='javascript:history.go(-1);')
        
        try:
            OpensslWrapper.AddPolicy(self.dir,REQUEST.form['policy'],REQUEST.form['template'])
        except:
            return MessageDialog(
                title  ='Error creating policy...',
                message="(%s): %s" % (sys.exc_type,sys.exc_value),
                action ='javascript:history.go(-1);')
        return MessageDialog(
            title  ='Policy created succesfully...',
            message='Policy created succesfully...',
            action ='policies')

    security.declarePublic('manage_enablePolicy')
    def manage_enablePolicy(self,REQUEST):
        """ enable policy """
        try:
            OpensslWrapper.EnablePolicy(self.dir,REQUEST.form['policy'])
        except:
            return MessageDialog(
                title  ='Error enabling policy...',
                message="(%s): %s" % (sys.exc_type,sys.exc_value),
                action ='policies')
        return MessageDialog(
            title  ='Policy enabled succesfully...',
            message='Policy enabled succesfully...',
            action ='policies')    

    security.declarePublic('manage_disablePolicy')
    def manage_disablePolicy(self,REQUEST):
        """ enable policy """
        try:
            OpensslWrapper.DisablePolicy(self.dir,REQUEST.form['policy'])
        except:
            return MessageDialog(
                title  ='Error disabling policy...',
                message="(%s): %s" % (sys.exc_type,sys.exc_value),
                action ='policies')
        return MessageDialog(
            title  ='Policy disabled succesfully...',
            message='Policy disabled succesfully...',
            action ='policies')

    security.declarePublic('manage_deletePolicy')
    def manage_deletePolicy(self,REQUEST):
        """ delete policy """
        try:
            OpensslWrapper.DeletePolicy(self.dir,REQUEST.form['policy'])
        except:
            return MessageDialog(
                title  ='Error deleting policy...',
                message="(%s): %s" % (sys.exc_type,sys.exc_value),
                action ='policies')
        return MessageDialog(
            title  ='Policy deleted succesfully...',
            message='Policy deleted succesfully...',
            action ='policies')

    security.declarePublic('manage_updatePolicy')
    def manage_updatePolicy(self,REQUEST):
        """ update policy """
        def FillDictFromRequest(dict,extension,request):

            if not dict[extension].has_key('extension'):
                if dict[extension]['type']['type'] == 'number':
                    dict[extension]['value'] = int(request[extension])
                elif dict[extension]['type']['type'] == 'string':
                    dict[extension]['value'] = request[extension]
                else:
                    raise "FormError","Extension type %s not valid" % dict[extension]['type']['type']
            else:
            
                # extension critical, present or notpresent
                if request[extension+'Option'] not in ['critical','notpresent','present']:
                    raise "FormError","value of field %s not valid (%s)" % (extension+'Option',request[extension+'Option'])
                dict[extension]['extension'] = request[extension+'Option']

                if dict[extension]['type']['type'] == 'dict':
                    for k in dict[extension]['value'].keys():
                        if type(dict[extension]['value'][k]) == types.DictType:
                            # INICIO PRESENT
                            if request.form.has_key(extension):
                                if k in request.form[extension]:
                                    dict[extension]['value'][k]['present'] = 'true'
                                else:
                                    dict[extension]['value'][k]['present'] = 'false'
                            else:
                                if request.form.has_key(extension+'_'+k):
                                    if request.form[extension+'_'+k] == 'true':
                                        dict[extension]['value'][k]['present'] = 'true'
                                    elif request.form[extension+'_'+k] == 'false':
                                        dict[extension]['value'][k]['present'] = 'false'
                                else:
                                    if request.form.has_key(extension+'_'+k+'_value'):
                                        dict[extension]['value'][k]['present'] = 'true'
                                    else:
                                        dict[extension]['value'][k]['present'] = 'false'
                            # FIN PRESENT
                            # INICIO VALUE
                            if dict[extension]['value'][k]['type']['type'] == 'boolean':
                                # BOOLEAN
                                if request.form.has_key(extension+'_'+k+'_value'):
                                    if request.form[extension+'_'+k+'_value'] == 'true':
                                        dict[extension]['value'][k]['value'] = 'true'
                                    elif request.form[extension+'_'+k+'_value'] == 'false':
                                        dict[extension]['value'][k]['value'] = 'false'
                                    else:
                                        raise "FormError","Extension value %s has not valid value(%s)" % (extension+k+'value',request.form[extension+k])
                                else:
                                    dict[extension]['value'][k]['value'] = dict[extension]['value'][k]['present']
                            elif dict[extension]['value'][k]['type']['type'] == 'number':
                                # NUMBER
                                dict[extension]['value'][k]['value'] = request.form[extension+'_'+k+'_value']
                            elif dict[extension]['value'][k]['type']['type'] == 'enum':
                                # ENUM
                                if request.form.has_key(extension+'_'+k+'_value'):
                                    dict[extension]['value'][k]['value'] = request.form[extension+'_'+k+'_value']
                                else:
                                    dict[extension]['value'][k]['value'] = None
                            elif dict[extension]['value'][k]['type']['type'] == 'string':
                                # STRING
                                dict[extension]['value'][k]['value'] = request.form[extension+'_'+k+'_value']
                        else:
                            # true or false hack
                            if request.form.has_key(extension):
                                if k in request.form[extension]:
                                    dict[extension]['value'][k] = 'true'
                                else:
                                    dict[extension]['value'][k] = 'false'
                            else:
                                dict[extension]['value'][k] = 'false'
                elif dict[extension]['type']['type'] == 'choice':
                    for k in dict[extension]['value'].keys():
                        if request.form.has_key(extension):
                            if request.form[extension] != '':
                                if k == request.form[extension]:
                                    dict[extension]['value'][k] = 'true'
                                else:
                                    dict[extension]['value'][k] = 'false'
                elif dict[extension]['type']['type'] == 'string':
                    dict[extension]['value'] = request.form[extension]
                else:
                    raise "FormError","Extension type %s not valid" % dict[extension]['type']['type']

            
        dict = OpensslWrapper.NewEmptyPolicy()

        FillDictFromRequest(dict,OpensslWrapper.OPTION_NOEXT_VALIDITY,REQUEST)
        FillDictFromRequest(dict,OpensslWrapper.OPTION_NOEXT_BITSDOWN,REQUEST)
        FillDictFromRequest(dict,OpensslWrapper.OPTION_NOEXT_BITSUP,REQUEST)
        
        FillDictFromRequest(dict,OpensslWrapper.OPTION_EXT_BASICCONSTRAINTS,REQUEST)
        FillDictFromRequest(dict,OpensslWrapper.OPTION_EXT_KEYUSAGE,REQUEST)
        FillDictFromRequest(dict,OpensslWrapper.OPTION_EXT_EXTENDEDKEYUSAGE,REQUEST)
        FillDictFromRequest(dict,OpensslWrapper.OPTION_EXT_SUBJECTKEYIDENTIFIER,REQUEST)
        FillDictFromRequest(dict,OpensslWrapper.OPTION_EXT_AUTHORITYKEYIDENTIFIER,REQUEST)
        FillDictFromRequest(dict,OpensslWrapper.OPTION_EXT_SUBJECTALTNAME,REQUEST)
        FillDictFromRequest(dict,OpensslWrapper.OPTION_EXT_ISSUERALTNAME,REQUEST)
        FillDictFromRequest(dict,OpensslWrapper.OPTION_EXT_NSCERTTYPE,REQUEST)
        FillDictFromRequest(dict,OpensslWrapper.OPTION_EXT_NSCOMMENT,REQUEST)
        FillDictFromRequest(dict,OpensslWrapper.OPTION_EXT_NSCAPOLICYURL,REQUEST)
        FillDictFromRequest(dict,OpensslWrapper.OPTION_EXT_NSCAREVOCATIONURL,REQUEST)
        FillDictFromRequest(dict,OpensslWrapper.OPTION_EXT_NSRENEWALURL,REQUEST)
        FillDictFromRequest(dict,OpensslWrapper.OPTION_EXT_CRLDISTRIBUTIONPOINTS,REQUEST)

        try:
            OpensslWrapper.UpdatePolicy(self.dir,REQUEST.form['orig_policy'],REQUEST.form['policy'],dict)
        except:
            return MessageDialog(
                title = 'Error updating policy...',
                message ="(%s): %s" % (sys.exc_type,sys.exc_value),
                action ='javascript:history.go(-1);')
        
        return MessageDialog(
            title  ='Policy updated succesfully...',
            message='Policy updated succesfully...',
            action ='policies')

    security.declarePublic('manage_certifyRequest')
    def manage_certifyRequest(self,REQUEST):
        """ certify request """


        cgi_values = [['request',REQUEST.form['request']]]
        url_requests = string.split(REQUEST.form['url_requests'],'&')
        for var in url_requests:
            pair = string.split(var,'=')
            if len(pair) == 1:
                cgi_values.append([pair,''])
            else:
                cgi_values.append([pair[0],pair[1]])

                
        ra = self.GetRaFolder()
        info_request = ra.GetInfoRequest(REQUEST.form['request'],not_html=1)

        if info_request['type'] == 'spkac':
            request_spkac = 1
        else:
            request_spkac = 0

        try:
            ra.CreateCertObject4Req(REQUEST.form['request'])
        except:
            get_transaction().abort()
            return MessageDialog(
                title = 'Error certificating request...',
                message ="Can not create certificate (maybe this request has been certificated before)<br>(%s): %s" % (sys.exc_type,sys.exc_value),
                action ='view_request',
                cgi_values=cgi_values)

        
        tempfilename = tempfile.mktemp()
        file = open(tempfilename,'w')
        file.write(info_request['content'])
        file.close()

        try:
            cert = OpensslWrapper.CertifyRequest(self.dir,self._getPassword(),request_spkac,tempfilename,info_request['policy'])
        except:
            get_transaction().abort()
            ra.SetRequestError(info_request['id'],"(%s): %s" % (sys.exc_type,sys.exc_value))
            return MessageDialog(
                title = 'Error certificating request...',
                message ="(%s): %s" % (sys.exc_type,sys.exc_value),
                action ='view_request',
                cgi_values=cgi_values)
        

        ra.SaveCert4Req(REQUEST.form['request'],OpensslWrapper.Certpem2Certder(cert))
                
        return MessageDialog(
            title  ='Certification request...',
            message=cert,
            action ='view_request',
            cgi_values=cgi_values)

    security.declarePublic('GetGeneratedCertificateList')
    def GetGeneratedCertificateList(self,fromIndex,count,status_req):
        if fromIndex <= 0:
            fromIndex = 0

        if count <= 0:
            count = 1

       
        try:
            certList = OpensslWrapper.GetCertificateList(self.dir,self._getPassword(),fromIndex,count,status_req)
        except:
            return MessageDialog(
                title = "Error getting CA certificate list",
                message ="(%s): %s" % (sys.exc_type,sys.exc_value),
                action ='certs')

        return certList

    security.declarePublic('GetIsssuedCertInfo')
    def GetIssuedCertInfo(self,serial):
        cert_info = {}
        cert_info['serial'] = serial

        try:
            cert_info['content'] = OpensslWrapper.GetIssuedCertificateInfo(self.dir,serial)
            lines = string.split(cert_info['content'],'\n')
            for l in range(len(lines)-1):
                if (len(lines[l]) > 0):
                    if lines[l][0] == ' ':
                        counter = 1
                        while counter < len(lines[l]):
                            if lines[l][counter] != ' ':
                                break
                            counter = counter + 1
                        for i in range(0,counter,2):
                            lines[l] =  '&nbsp;' + lines[l]
            cert_info['content'] = string.join(lines,'<br>\n')
        except:
            return MessageDialog(
                title = "Error getting CA certificate list",
                message ="(%s): %s" % (sys.exc_type,sys.exc_value),
                action ='certs')
        
        return cert_info

    security.declarePublic('manage_revokeCertificate')
    def manage_revokeCertificate(self,REQUEST):
        """revoke issued certificate"""

        cgi_values = [['serial',REQUEST.form['serial']]]
        url_certs = string.split(REQUEST.form['url_certs'],'&')
        for var in url_certs:
            pair = string.split(var,'=')
            if len(pair) == 1:
                cgi_values.append([pair,''])
            else:
                cgi_values.append([pair[0],pair[1]])
                
        instruction_param = None
        time_param       = None
        if REQUEST.form.has_key('time'):
            time_param = REQUEST.form['time']
        if REQUEST.form.has_key('instruction'):
            instruction_param = REQUEST.form['instruction']
            
        try:
            OpensslWrapper.RevokeIssuedCert(self.dir,self._getPassword(),REQUEST.form['serial'],REQUEST.form['reason'],time_param,instruction_param)
        except:
            return MessageDialog(
                title = 'Error revoking certificate...',
                message = "(%s): %s" % (sys.exc_type,sys.exc_value),
                action="view_issued_cert",
                cgi_values=cgi_values)

        return MessageDialog(
            title  ='Certificate revoked...',
            message='certificate with serial %s has been revoked' % REQUEST.form['serial'],
            action ='view_issued_cert',
            cgi_values=cgi_values)
    
    security.declarePublic('manage_exportKey')    
    def manage_exportKey(self,REQUEST):
        """ manage_exportKey """
        return MessageDialog(
            title = 'Method not implemented',
            message = 'METHOD NOT IMPLENTED',
            action='javascript:history.go(-1);')

    security.declarePublic('manage_exportCert')    
    def manage_exportCert(self,cert,own=None,pem=None,REQUEST=None):
        """ manage_exportCert """
        RESPONSE = REQUEST.response
        try:
            certificate_pem = OpensslWrapper.GetCACert(self.dir,self._getPassword(),cert)
            if certificate_pem is None:
                return MessageDialog(
                    title = 'Error getting certificate',
                    message = 'Certificate does not exist',
                    action='javascript:history.go(-1);')
            if pem is not None:
                certificate = certificate_pem
                filename    = "certificate.pem"
            else:
                certificate = OpensslWrapper.Certpem2Certder(certificate_pem)
                filename = "certificate.cer"
                
            RESPONSE.setHeader('Content-Type', 'application/unknown-application;name=%s'%filename)
            RESPONSE.setHeader('Content-Length', len(certificate))
            RESPONSE.setHeader('Content-Disposition','inline;filename=%s' % filename)
            RESPONSE.write(certificate)
        except:
            return MessageDialog(
                title = 'Error getting certificate...',
                message = "(%s): %s" % (sys.exc_type,sys.exc_value),
                action="javascript:history.go(-1);")
        

    security.declarePublic('manage_generateRequest')    
    def manage_generateRequest(self,x509=None,own=None,cert=None,REQUEST=None):
        """ manage_generateRequest """
        # generate always a request from CA ONLY key
        RESPONSE = REQUEST.response
        if x509 is not None:
            x509 = 1
            filename = "x509_req.pem"
        else:
            filename = "req.pem"
            x509 = 0
        try:
            request_pem = OpensslWrapper.GetCARequest(self.dir,self._getPassword(),x509)
            RESPONSE.setHeader('Content-Type', 'application/unknown-application;name=%s'%filename)
            RESPONSE.setHeader('Content-Length', len(request_pem))
            RESPONSE.setHeader('Content-Disposition','inline;filename=%s' % filename)
            RESPONSE.write(request_pem)
        except:
            return MessageDialog(
                title = 'Error generating request...',
                message = "(%s): %s" % (sys.exc_type,sys.exc_value),
                action="javascript:history.go(-1);")
        


    security.declarePublic('manage_deleteCert')    
    def manage_deleteCert(self,cert,own=None,REQUEST=None):
        """ manage_deleteCert """
        if own is not None and own == "1":
            return MessageDialog(
                title = 'Error removing a certificate',
                message = 'You can not remove CA cert',
                action = 'keys')
        try:
            message = OpensslWrapper.RemoveCACert(self.dir,self._getPassword(),cert)
            return MessageDialog(
                title = 'Certificate removed',
                message = message,
                action='keys')
        except:
            return MessageDialog(
                 title  ='Error removing a certificate',
                 message= "(%s): %s" % (sys.exc_type,sys.exc_value),
                 action ='keys')

    security.declarePublic('manage_denyRequest')    
    def manage_denyRequest(self,REQUEST):
        """ manage_denyRequest """
        return MessageDialog(
            title = 'Method not implemented',
            message = 'METHOD NOT IMPLENTED',
            action='javascript:history.go(-1);')

    security.declarePublic('manage_requestAction')    
    def manage_requestAction(self,REQUEST):
        """ manage_requestAction """
        return MessageDialog(
            title = 'Method not implemented',
            message = 'METHOD NOT IMPLENTED',
            action='javascript:history.go(-1);')

    security.declarePublic('manage_certAction')    
    def manage_certAction(self,REQUEST):
        """ manage_certAction """
        return MessageDialog(
            title = 'Method not implemented',
            message = 'METHOD NOT IMPLENTED',
            action='javascript:history.go(-1);')

    security.declarePublic('GetCrlInfo')
    def GetCrlInfo(self):
        """ Get CRL info """
        info = OpensslWrapper.GetCrlInfo(self.dir)
        return info

    security.declarePublic('GetCrlExtendedInfo')
    def GetCrlExtendedInfo(self):
        """ Get CRL info """
        info = OpensslWrapper.GetCrlExtendedInfo(self.dir)
        return info

    security.declarePublic('CrlIsExpired')
    def CrlIsExpired(self):
        """ CRL is expired """
        info = OpensslWrapper.GetCrlInfo(self.dir)
        if not info.has_key('nextUpdate'):
          return 1
        tnow = time.time()
        tcrl = time.mktime(info['nextUpdate'])
        #raise "ERROR","%d>=%d" % (tnow,tcrl)
        if tnow >= tcrl:
            return 1
        return 0

    security.declarePublic('manage_generateNewCrl')
    def manage_generateNewCrl(self):
        """ Generate New Crl """
        try:
            ret = OpensslWrapper.GenerateNewCrl(self.dir,self._getPassword())
        except:
            return MessageDialog(
                 title  ='Error generating new CRL...',
                 message= "(%s): %s" % (sys.exc_type,sys.exc_value),
                 action ='crls')
        
        return MessageDialog(
                title  ='CRL generated successfully',
                message="CRL generated successfully",
                action ='crls')

    
    security.declarePublic('FormatTime')
    def FormatTime(self,s_t=None):
        """ FormatTime """
        if s_t is None:
            s_t = time.gmtime()
        return time.strftime('%a, %d %b %Y %H:%M:%S +0000', s_t)

    security.declarePublic('manage_publishCRLandCAcerts')
    def manage_publishCRLandCAcerts(self):
        """ Publish CRL an CAcerts """
        try:
            crl_pem = OpensslWrapper.GetCRL(self.dir,"pem")
            p7_pem = OpensslWrapper.GetP7CAOwnCertificate(self.dir,type="pem")
            ra = self.GetRaFolder()
            ra.PublishCRLandCAcerts(crl_pem,p7_pem)
        except:
            return MessageDialog(
                title  ='Publishing CRL an CA certs',
                 message= "(%s): %s" % (sys.exc_type,sys.exc_value),
                 action ='publish')
        
        return MessageDialog(
                title  ='CRL and CA certs published successfully',
                message="CRL and CA certs published successfully",
                action ='publish')
            

    security.declarePublic('manage_setCRLGenPeriod')
    def manage_setCRLGenPeriod(self,value_type=None,crl_days=None,crl_hours=None,REQUEST=None):
        """ Set CRL gen period """
        try:
            crl_days_value = None
            crl_hours_value = None
            if value_type == 'crl_days':
                crl_days_value = crl_days
            else:
                crl_hours_value = crl_hours
                
            OpensslWrapper.SetCRLGenPeriod(self.dir,crl_days_value,crl_hours_value)
        except:
            return MessageDialog(
                title  ='Error setting CRL gen period',
                message= "(%s): %s" % (sys.exc_type,sys.exc_value),
                action ='crls')

        return MessageDialog(
                title  ='CRL gen period successfully changed',
                message="CRL gen period successfully changed",
                action ='crls')

    security.declarePublic("GetCRLGenPeriod")
    def GetCRLGenPeriod(self,REQUEST=None):
        return OpensslWrapper.GetCRLGenPeriod(self.dir)
    

InitializeClass(SecClabCa)





